# Created by Noah Kantrowitz on 2007-04-03.
# Copyright (c) 2007 Noah Kantrowitz. All rights reserved.

from trac.core import *
from trac.web.chrome import ITemplateProvider
from trac.ticket.api import TicketSystem
from webadmin.web_ui import IAdminPageProvider

class HideValsAdminModule(Component):
    """WebAdmin sub-page for configuring the TracHideVals plugins."""

    implements(IAdminPageProvider, ITemplateProvider)
    
    # IAdminPageProvider methods
    def get_admin_pages(self, req):
        if req.perm.has_permission('TICKET_ADMIN'):
            for field in TicketSystem(self.env).get_ticket_fields():
                if 'options' in field:
                    yield 'hidevals', 'Hide Values', field['name'], field['label']

    def process_admin_request(self, req, cat, page, path_info):
        db = self.env.get_db_cnx()
        cursor = db.cursor()

        field = dict([(field['name'], field) for field in TicketSystem(self.env).get_ticket_fields()])[page]
        cursor.execute('SELECT sid, value FROM hidevals WHERE field = %s', (field['name'],))
        values = cursor.fetchall()
        
        if req.method == 'POST':
            if req.args.get('add'):
                group = req.args['group']
                value = req.args['value']
                if (group, value) not in values:
                    cursor.execute('INSERT INTO hidevals (sid, field, value) VALUES (%s, %s, %s)', (group, field['name'], value))
                    db.commit()
            elif req.args.get('remove'):
                sel = req.args.getlist('sel')
                for val in sel:
                    group, value = val.split('#', 1)
                    cursor.execute('DELETE FROM hidevals WHERE sid = %s AND field = %s AND value = %s', (group, field['name'], value))
                    db.commit()
            req.redirect(req.href.admin(cat, page))
        
        req.hdf['hidevals.field'] = field
        req.hdf['hidevals.values'] = [{'group': g, 'value': v} for g,v in values]
        return 'admin_hidevals.cs', None

    # ITemplateProvider methods
    def get_templates_dirs(self):
        from pkg_resources import resource_filename
        return [resource_filename(__name__, 'templates')]
         
    def get_htdocs_dirs(self):
        from pkg_resources import resource_filename
        return [('hidevals', resource_filename(__name__, 'htdocs'))]



# Created by Noah Kantrowitz on 2007-04-02.
# Copyright (c) 2007 Noah Kantrowitz. All rights reserved.

from trac.db import Table, Column

name = 'hidevals'
version = 1
tables = [
    Table('hidevals', key=('sid', 'field', 'value'))[
        Column('sid'),
        Column('field'),
        Column('value'),
    ],   
]


# Created by Noah Kantrowitz on 2007-04-02.
# Copyright (c) 2007 Noah Kantrowitz. All rights reserved.

from trac.core import *
from trac.web.api import IRequestFilter

try:
    set = set
except NameError:
    from sets import Set as set

from api import HideValsSystem

class HideValsFilter(Component):
    """A filter to hide certain ticket field values."""

    implements(IRequestFilter)

    # IRequestFilter methods
    def pre_process_request(self, req, handler):
        return handler
            
    def post_process_request(self, req, template, content_type):
        if req.perm.has_permission('TRAC_ADMIN'):
            # TRAC_ADMIN would have the filterer permissions by inheritance
            return template, content_type
            
        if req.perm.has_permission('TICKET_HIDEVALS'):
            fields = HideValsSystem(self.env).visible_fields(req)
            to_delete = []
            
            base = None
            if req.path_info.startswith('/newticket'):
                base = 'newticket'
            elif req.path_info.startswith('/ticket'):
                base = 'ticket'
                
            if base:
                sub_hdf = req.hdf.getObj(base+'.fields').child()
                while sub_hdf:
                    self.log.debug('HideVals: !')
                    if sub_hdf.getObj('options'):
                        if sub_hdf.name() in fields:
                            # If we have any values, filter what is there
                            ###req.hdf.removeTree('newticket.fields.%s.options'%sub_hdf.name())
                            ###req.hdf['newticket.fields.%s.options'%sub_hdf.name()] = fields[sub_hdf.name()]
                            # ???: Can a field have no options at this point? <NPK>
                            opts = sub_hdf.getObj('options').child() 
                            valid_opts = set(fields[sub_hdf.name()])
                            opts_to_delete = []
                            while opts:
                                if opts.value() not in valid_opts:
                                    opts_to_delete.append(opts.name())
                                opts = opts.next()
                            
                            for opt in opts_to_delete:
                                req.hdf.removeTree('%s.fields.%s.options.%s'%(base, sub_hdf.name(), opt))
                        else:
                            # If there are no values for this user, remove the field entirely
                            # NOTE: Deleting in place screws up the iteration, so do it all afterwards. <NPK>
                            to_delete.append(sub_hdf.name())
                        
                    sub_hdf = sub_hdf.next()
                    
                for field in to_delete:
                    req.hdf.removeTree(base+'.fields.'+field)
                        
        return template, content_type


#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

from setuptools import setup

setup(
    name = 'TracHideVals',
    version = '1.0',
    packages = ['hidevals'],
    package_data = { 'hidevals': ['templates/*.cs', 'htdocs/*.js', 'htdocs/*.css' ] },

    author = 'Noah Kantrowitz',
    author_email = 'coderanger@yahoo.com',
    description = 'Hide ticket option values from certain users.',
    license = 'BSD',
    keywords = 'trac plugin',
    url = 'http://trac-hacks.org/wiki/HideValsPlugin',
    classifiers = [
        'Framework :: Trac',
    ],
    
    install_requires = ['TracWebAdmin'],

    entry_points = {
        'trac.plugins': [
            'hidevals.filter = hidevals.filter',
            'hidevals.api = hidevals.api',
            'hidevals.admin = hidevals.admin',
        ]
    },
)



# Created by Noah Kantrowitz on 2007-04-02.
# Copyright (c) 2007 Noah Kantrowitz. All rights reserved.

from trac.core import *
from trac.env import IEnvironmentSetupParticipant
from trac.perm import IPermissionRequestor, IPermissionGroupProvider, PermissionSystem

try:
    set = set
except NameError:
    from sets import Set as set

import db_default

class HideValsSystem(Component):
    """Database provider for the TracHideVals plugin."""

    group_providers = ExtensionPoint(IPermissionGroupProvider)

    implements(IPermissionRequestor, IEnvironmentSetupParticipant)
    
    # Public methods
    def visible_fields(self, req, db=None):
        db = db or self.env.get_db_cnx()
        cursor = db.cursor()
        
        groups = self._get_groups(req.authname)
        fields = {}
        for group in groups:
            cursor.execute('SELECT field, value FROM hidevals WHERE sid = %s', (group,))
            for f, v in cursor:
                fields.setdefault(f, []).append(v)
                
        return fields
        
    # IPermissionRequestor methods
    def get_permission_actions(self):
        yield 'TICKET_HIDEVALS'

    # IEnvironmentSetupParticipant methods
    def environment_created(self):
        self.found_db_version = 0
        self.upgrade_environment(self.env.get_db_cnx())
        
    def environment_needs_upgrade(self, db):
        cursor = db.cursor()
        cursor.execute("SELECT value FROM system WHERE name=%s", (db_default.name,))
        value = cursor.fetchone()
        if not value:
            self.found_db_version = 0
            return True
        else:
            self.found_db_version = int(value[0])
            #self.log.debug('WeatherWidgetSystem: Found db version %s, current is %s' % (self.found_db_version, db_default.version))
            return self.found_db_version < db_default.version
            
    def upgrade_environment(self, db):
        # 0.10 compatibility hack (thanks Alec)
        try:
            from trac.db import DatabaseManager
            db_manager, _ = DatabaseManager(self.env)._get_connector()
        except ImportError:
                db_manager = db
                
        # Insert the default table
        old_data = {} # {table_name: (col_names, [row, ...]), ...}
        cursor = db.cursor()
        if not self.found_db_version:
            cursor.execute("INSERT INTO system (name, value) VALUES (%s, %s)",(db_default.name, db_default.version))
        else:
            cursor.execute("UPDATE system SET value=%s WHERE name=%s",(db_default.version, db_default.name))
            for tbl in db_default.tables:
                try:
                    cursor.execute('SELECT * FROM %s'%tbl.name)
                    old_data[tbl.name] = ([d[0] for d in cursor.description], cursor.fetchall())
                    cursor.execute('DROP TABLE %s'%tbl.name)
                except Exception, e:
                    if 'OperationalError' not in e.__class__.__name__:
                        raise e # If it is an OperationalError, just move on to the next table
                            
                
        for tbl in db_default.tables:
            for sql in db_manager.to_sql(tbl):
                cursor.execute(sql)
                    
            # Try to reinsert any old data
            if tbl.name in old_data:
                data = old_data[tbl.name]
                sql = 'INSERT INTO %s (%s) VALUES (%s)' % \
                  (tbl.name, ','.join(data[0]), ','.join(['%s'] * len(data[0])))
                for row in data[1]:
                    try:
                        cursor.execute(sql, row)
                    except Exception, e:
                        if 'OperationalError' not in e.__class__.__name__:
                            raise e
                            
    # Private methods
    def _get_groups(self, user):
        # Get initial subjects
        groups = set([user])
        for provider in self.group_providers:
            for group in provider.get_permission_groups(user):
                groups.add(group)

        perms = PermissionSystem(self.env).get_all_permissions()
        repeat = True
        while repeat:
            repeat = False
            for subject, action in perms:
                if subject in groups and action.islower() and action not in groups:
                    groups.add(action)
                    repeat = True 

        return groups